/*******************************************************************************

Copyright (c) 1997-2004, Perforce Software, Inc.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1.  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

2.  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTR
IBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

/*******************************************************************************
 * Name		: php_clientapi.h
 *
 * Author	: Tony Smith <tony@perforce.com> or <tony@smee.org>
 *
 * Description	: Php bindings for the Perforce API. Definitions of our
 * 		  main interface to Perforce
 *
 ******************************************************************************/


/*******************************************************************************
 * PhpClientApi class - where we register our Php classes and plumb together 
 * the components
 ******************************************************************************/

class PhpClientApi
{
    public:
    PhpClientApi();
    ~PhpClientApi();

    // Protocol options. Call before Connect()
    void Tagged();
    void ParseForms();

    void SetCharset( const char *c )	{ client.SetCharset( c );	}
    void SetClient( const char *c )	{ client.SetClient( c );	}
    void SetCwd( const char *c )	{ client.SetCwd( c ); 		}
    void SetHost( const char *h )	{ client.SetHost( h );		}
    void SetLanguage( const char *l )	{ client.SetLanguage( l );	}
    void SetPassword( const char *p )	{ client.SetPassword( p );	}
    void SetPort( const char *p )	{ client.SetPort( p );		}
    void SetProg( const char *p );
    void SetUser( const char *u )	{ client.SetUser( u );		}

    const StrPtr &GetCharset()		{ return client.GetCharset();	}
    const StrPtr &GetClient()		{ return client.GetClient();	}
    const StrPtr &GetCwd()		{ return client.GetCwd();	}
    const StrPtr &GetHost()		{ return client.GetHost();	}
    const StrPtr &GetLanguage()		{ return client.GetLanguage();	}
    const StrPtr &GetOs()		{ return client.GetOs();	}
    const StrPtr &GetPassword()		{ return client.GetPassword();	}
    const StrPtr &GetPort()		{ return client.GetPort();	}
    const StrPtr &GetUser()		{ return client.GetUser();	}

    // Session management
    int	  Connect();
    int   Disconnect();
    int   Dropped();

    // Executing commands. 
    zval* Run( const char *cmd, int argc, char * const *argv );
    void  SetInput( zval * input );

    // Result handling
    zval* GetOutput()		{ return ui.GetResults().GetOutput();}
    zval* GetErrors()		{ return ui.GetResults().GetErrors();}
    zval* GetWarnings()		{ return ui.GetResults().GetWarnings();}

    // Spec parsing
    zval* ParseSpec( const char * type, const char *form );

    //
    // Debugging support. Debug levels are:
    //
    //     1:	Debugs commands being executed
    //     2:	Debug UI method calls
    //     3:	Show garbage collection
    //
    void  SetDebug( int d );

    private:

    void RunCmd(const char *cmd, ClientUser *ui, int argc, char * const *argv);
    StrPtr * 	FetchSpecDef( const char *type );
    void   	ReportError( Error *e );
    void	ReportError( const char *msg );

    public:
    ClientApi		client;

    private:
    PhpClientUser	ui;
    StrBufDict		specDict;
    int			initCount;
    int			debug;
    int			server2;
    int			mode;
};

